/**
 * React Blessed Component
 * ========================
 *
 * React component abstraction for the blessed library.
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _blessed = require('blessed');

var _blessed2 = _interopRequireDefault(_blessed);

var _reactLibReactMultiChild = require('react/lib/ReactMultiChild');

var _reactLibReactMultiChild2 = _interopRequireDefault(_reactLibReactMultiChild);

var _ReactBlessedIDOperations = require('./ReactBlessedIDOperations');

var _ReactBlessedIDOperations2 = _interopRequireDefault(_ReactBlessedIDOperations);

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

var _update = require('./update');

var _update2 = _interopRequireDefault(_update);

var _solveClass = require('./solveClass');

var _solveClass2 = _interopRequireDefault(_solveClass);

var _lodash = require('lodash');

/**
 * Renders the given react element with blessed.
 *
 * @constructor ReactBlessedComponent
 * @extends ReactMultiChild
 */

var ReactBlessedComponent = (function () {
  function ReactBlessedComponent(tag) {
    _classCallCheck(this, ReactBlessedComponent);

    this._tag = tag.toLowerCase();
    this._renderedChildren = null;
    this._previousStyle = null;
    this._previousStyleCopy = null;
    this._rootNodeID = null;
    this._wrapperState = null;
    this._topLevelWrapper = null;
    this._nodeWithLegacyProperties = null;
  }

  /**
   * Extending the component with the MultiChild mixin.
   */

  _createClass(ReactBlessedComponent, [{
    key: 'construct',
    value: function construct(element) {

      // Setting some properties
      this._currentElement = element;
    }

    /**
     * Mounting the root component.
     *
     * @internal
     * @param  {string} rootID - The root blessed ID for this node.
     * @param  {ReactBlessedReconcileTransaction} transaction
     * @param  {object} context
     */
  }, {
    key: 'mountComponent',
    value: function mountComponent(rootID, transaction, context) {
      this._rootNodeID = rootID;

      // Mounting blessed node
      var node = this.mountNode(_ReactBlessedIDOperations2['default'].getParent(rootID), this._currentElement);

      _ReactBlessedIDOperations2['default'].add(rootID, node);

      // Mounting children
      var childrenToUse = [].concat(this._currentElement.props.children || []);

      this.mountChildren(childrenToUse, transaction, context);

      // Rendering the screen
      _ReactBlessedIDOperations2['default'].screen.debouncedRender();
    }

    /**
     * Mounting the blessed node itself.
     *
     * @param   {BlessedNode|BlessedScreen} parent  - The parent node.
     * @param   {ReactElement}              element - The element to mount.
     * @return  {BlessedNode}                       - The mounted node.
     */
  }, {
    key: 'mountNode',
    value: function mountNode(parent, element) {
      var _this = this;

      var props = element.props;
      var type = element.type;
      var children = props.children;
      var options = _objectWithoutProperties(props, ['children']);
      var blessedElement = _blessed2['default'][type];

      (0, _reactLibInvariant2['default'])(!!blessedElement, 'Invalid blessed element "' + type + '".');

      var node = _blessed2['default'][type]((0, _solveClass2['default'])(options));

      node.onAny(function (type) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        var handler = _this._currentElement.props['on' + (0, _lodash.startCase)(type)];

        if (typeof handler === 'function') handler.apply(null, args);
      });

      parent.append(node);

      return node;
    }

    /**
     * Receive a component update.
     *
     * @param {ReactReconcileTransaction} transaction
     * @param {ReactElement}              prevElement
     * @param {ReactElement}              nextElement
     * @param {object}                    context
     * @internal
     * @overridable
     */
  }, {
    key: 'receiveComponent',
    value: function receiveComponent(nextElement, prevElement, transaction, context) {
      var _nextElement$props = nextElement.props;
      var children = _nextElement$props.children;
      var options = _objectWithoutProperties(_nextElement$props, ['children']);
      var node = _ReactBlessedIDOperations2['default'].get(this._rootNodeID);

      (0, _update2['default'])(node, (0, _solveClass2['default'])(options));

      // Updating children
      var childrenToUse = [].concat(children || []);

      this.updateChildren(childrenToUse, transaction, context);

      _ReactBlessedIDOperations2['default'].screen.debouncedRender();
    }

    /**
     * Dropping the component.
     */
  }, {
    key: 'unmountComponent',
    value: function unmountComponent() {
      this.unmountChildren();

      var parent = _ReactBlessedIDOperations2['default'].getParent(this._rootNodeID),
          node = _ReactBlessedIDOperations2['default'].get(this._rootNodeID);

      node.offAny();
      parent.remove(node);

      _ReactBlessedIDOperations2['default'].drop(this._rootNodeID);

      this._rootNodeID = null;
    }

    /**
     * Getting a public instance of the component for refs.
     *
     * @return {BlessedNode} - The instance's node.
     */
  }, {
    key: 'getPublicInstance',
    value: function getPublicInstance() {
      return _ReactBlessedIDOperations2['default'].get(this._rootNodeID);
    }
  }]);

  return ReactBlessedComponent;
})();

exports['default'] = ReactBlessedComponent;
(0, _lodash.extend)(ReactBlessedComponent.prototype, _reactLibReactMultiChild2['default'].Mixin);
module.exports = exports['default'];