/**
 * React Blessed
 * ==============
 *
 * Exposing the renderer's API.
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactLibReactInstanceHandles = require('react/lib/ReactInstanceHandles');

var _reactLibReactInstanceHandles2 = _interopRequireDefault(_reactLibReactInstanceHandles);

var _reactLibReactElement = require('react/lib/ReactElement');

var _reactLibReactElement2 = _interopRequireDefault(_reactLibReactElement);

var _reactLibReactUpdates = require('react/lib/ReactUpdates');

var _reactLibReactUpdates2 = _interopRequireDefault(_reactLibReactUpdates);

var _ReactBlessedIDOperations = require('./ReactBlessedIDOperations');

var _ReactBlessedIDOperations2 = _interopRequireDefault(_ReactBlessedIDOperations);

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

var _reactLibInstantiateReactComponent = require('react/lib/instantiateReactComponent');

var _reactLibInstantiateReactComponent2 = _interopRequireDefault(_reactLibInstantiateReactComponent);

var _ReactBlessedInjection = require('./ReactBlessedInjection');

var _ReactBlessedInjection2 = _interopRequireDefault(_ReactBlessedInjection);

var _blessed = require('blessed');

/**
 * Injecting dependencies.
 */

var _blessed2 = _interopRequireDefault(_blessed);

(0, _ReactBlessedInjection2['default'])();

/**
 * Renders the given react element with blessed.
 *
 * @param  {ReactElement}  element   - Node to update.
 * @param  {object}        [opts={}] - Options to give to the blessed screen.
 * @return {BlessedScreen}           - The created blessed screen.
 */
function render(element) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  // Is the given element valid?
  (0, _reactLibInvariant2['default'])(_reactLibReactElement2['default'].isValidElement(element), 'render(): You must pass a valid ReactElement.');

  // Creating a root id & creating the screen
  var id = _reactLibReactInstanceHandles2['default'].createReactRootID(),
      screen = _blessed2['default'].screen(opts);

  // Mounting the app
  var transaction = _reactLibReactUpdates2['default'].ReactReconcileTransaction.getPooled(),
      component = (0, _reactLibInstantiateReactComponent2['default'])(element);

  // Injecting the screen
  _ReactBlessedIDOperations2['default'].setScreen(screen);

  transaction.perform(function () {
    component.mountComponent(id, transaction, {});
  });

  // Returning the screen so the user can attach listeners etc.
  return screen;
}

exports.render = render;